#ifndef IndexSetForQueryingStage_h
#define IndexSetForQueryingStage_h


#include "../common/bserialize.h"
/////////////////////////////////////////////////////////////////////////////
// Flags for CIndexItem::m_IndexItemOffsetAndFlags

const DWORD TheOnlyOccurIsInEndOccurNo = 1<<31;
const DWORD EndOfSentenceForBigrams = 1<<30;
const DWORD AllFlags = TheOnlyOccurIsInEndOccurNo  | EndOfSentenceForBigrams;

/*!
	CIndexItem contains basic information about one index item. In the strict sense one index item
	is a connection between an indexed string and its occurrences. More generally the term "index item"
	is used to refer also an indexed string.
*/
class CIndexItem 
{
	// pointer to CStringIndexSet::m_StringBuffer and flags;
	DWORD				m_IndexItemOffsetAndFlags;

	// upper bound offset of occurrences in CIndexSetForQueryingStage::m_OccursFp
	DWORD				m_EndOccurOffset;

public:
	//! return m_IndexItemOffsetAndFlags for serialization
	DWORD	GetIndexItemOffsetAndFlags() const
	{
		return m_IndexItemOffsetAndFlags;
	};
	//! set m_IndexItemOffsetAndFlags for serialization
	void	SetIndexItemOffsetAndFlags(DWORD Value) 
	{
		m_IndexItemOffsetAndFlags = Value;
	};

	static size_t GetMaximalNumberOfRunningTokens()
	{
		return ~0 & ~AllFlags; 
	};

	//! returns a reference of this index item to CStringIndexSet::m_StringBuffer
	DWORD	GetIndexItemOffset() const
	{
		return m_IndexItemOffsetAndFlags & ~AllFlags;
	};
	
	//! sets a reference  of this index item to CStringIndexSet::m_StringBuffer
	void	SetIndexItemOffset(DWORD Value) 
	{
        m_IndexItemOffsetAndFlags &= AllFlags;
		m_IndexItemOffsetAndFlags |= Value & ~AllFlags;
	};
	//! returns  flags of  this  index item
	DWORD	GetItemIndexFlags() const
	{
		return m_IndexItemOffsetAndFlags & AllFlags;
	};
	//! set flags of  this  index item
	void	AddItemIndexFlags(DWORD Value) 
	{
		m_IndexItemOffsetAndFlags |= Value & AllFlags;
	};
	//! return the upper bound offset of occurrences in CIndexSetForQueryingStage::m_OccursFp
	DWORD GetEndOccurOffset() const 
	{
		return m_EndOccurOffset;
	};
	//! set the upper bound offset of occurrences in CIndexSetForQueryingStage::m_OccursFp
	void SetEndOccurOffset(DWORD EndOccurOffset)  
	{
		m_EndOccurOffset = EndOccurOffset;
	};
    bool HasOneOccurrence () const 
    {
        return (m_IndexItemOffsetAndFlags & TheOnlyOccurIsInEndOccurNo) != 0;
    }
};

inline size_t get_size_in_bytes (const CIndexItem& t)
{
	return 8;
};

inline size_t save_to_bytes(const CIndexItem& i, BYTE* buf)
{
	buf += save_to_bytes(i.GetIndexItemOffsetAndFlags(), buf);
	buf += save_to_bytes(i.GetEndOccurOffset(), buf);
	return get_size_in_bytes(i);
}

inline size_t restore_from_bytes(CIndexItem& i, const BYTE* buf)
{
	DWORD d;

	buf += restore_from_bytes(d, buf);
	i.SetIndexItemOffsetAndFlags(d);

	buf += restore_from_bytes(d, buf);
	i.SetEndOccurOffset(d);

	return get_size_in_bytes(i);
}


class CStringIndexator; 
class CItemIndexForLoading;

/*!	This class is  a part of CStringIndexSet class which is used only during querying.  It contains
the important serialization primitives.
*/
class CIndexSetForQueryingStage 
{
	//! the main file of  occurrences 
	FILE*			m_OccursFp;
	
	
	//!		close m_OccursFp
	void	CloseOccursFile();
	//!		load all \ref perdiv_def "period divisions" to m_EndPeriodOffsets
	bool	LoadPeriodDevision();
	

protected:

	//!		return true, if the project path is initialized
	void	AssertHasPath() const;
	//!  a function for reading occurrences for one index item
	void	AddOccurs (size_t IndexItemNo, const bool bOneOccurrence, const size_t StartOccurNo, const size_t EndOccurNo, vector<CTokenNo>& Occurs, size_t PeriodNo, COccurrBuffer& OccursBuffer, CShortOccurCache* pCacheByIndexSet, int& CacheId) const;
	//!  return the file name  for the file occurrences
	string	GetOccursFileName() const;
	//!  return the name  of file for m_Index
	string	GetOccHdrFileName() const;
	
	
	//!  return the name  of file for occurrences \ref perdiv_def "period division"
	string	GetPeriodsDevisionFileName () const;
	//!  return the name  of file for CIndexSetForLoadingStage::m_StringBuffer
	string	GetFileNameForInfos() const;
	//! return the size of the file for occurrences
	file_off_t	GetOccurrsFileSize() const;
	//!  get the offset of the first occurrence of index item no IndexNo in the file of occurrences(m_OccursFp)
	size_t	GetStartOccurNo(size_t IndexNo) const;
   	//!		build  a \ref perdiv_def "period division"  for one index item  
	bool	BuildPeriodsDivisionAndCompress (const DWORD TokenId, vector<CTokenNo>& InputTokens);
	//! write one index item to result file
	bool	AddOneIndexItem (CItemIndexForLoading& M, FILE* res_fp, size_t& CurrPositionInResFile, const CTokenNo EndTokeNo);
	//! write index item's \ref perdiv_def "period division"  to disk
	bool	WritePeriodsDivision ();


public:
	//! the main index(from strings to the ordered list of their occurrences) 
	vector<CIndexItem>				m_Index;

	//! all corpus \ref perdiv_def "period divisions" for the long occurrence lists
	PeriodsDivisionMap			m_EndPeriodOffsets;

	//! a pointer to the collection of indices, which contains a reference to this index
	const CStringIndexator*		m_pParent;

	//! if true, then the occurrences should be compresses (up to 30% for huge corpora)
	bool						m_bCompressOccurrences;


	CIndexSetForQueryingStage(const CStringIndexator*		pParent);
	~CIndexSetForQueryingStage();
	//! return the name of the index (CStringIndexSet::m_Name)
	virtual string GetName() const = 0;
	//! load index set from binaries
	bool	LoadIndexSet(bool bLoadHeaderOfOccurrences = true);
	//! destroy index set and remove index files
	bool	DestroyIndexSet ();
	//! reads all occurrences of IndexItemNo (this function can allocate much memory; it should be used carefully)
	void ReadAllOccurrences (size_t IndexItemNo, vector<CTokenNo>& Occurs) const;
    
};



#endif
